@@ -9,6 +9,11 @@ try:
 except:
     pass
 
+try:
+    from case_insensitive_dict import CaseInsensitiveDict
+except:
+    pass
+
 '''
 License: GNU General Public License v3.0 (http://www.gnu.org/licenses/gpl-3.0.html)
 Author: Mr. You Duan
@@ -94,12 +99,15 @@ class Seq(str):
         sys.stderr.write("write2fasta will retired, use to_fasta replace\n")
         self.to_fasta(outfile,lth)
 
-class Fadict(dict):
+class FadictBase(object):
+    """ Dict of fasta file, provide functions. 
+        [seqname: seq]
+        :param: strict
+        :param: if_trim
+        :param: ignorecase
+    """
+    def init(self,infile,strict=False,ignorecase=False,**kwargs):
 
-    def __new__(self,*args,**kwargs):
-        return dict.__new__(self)
-
-    def __init__(self,infile,strict=False,**kwargs):
         faiter = Fasta(infile,**kwargs)
         self.filename, self.l_uplim, self.l_lowlim = \
         faiter.filename, faiter.l_uplim, faiter.l_lowlim
@@ -124,7 +132,10 @@ class Fadict(dict):
         return 1
 
     def _getSeq(self,scaf,st,ed):
-        scaf = self.getChr(scaf)
+        key = scaf
+        scaf = self.getChr(key)
+        if scaf == None:
+            raise KeyError("Scaf \"{}\" was not found, please check it.".format(key))
         #if st < 1 or ed < 1:
         #    raise ValueError("sequence is 1-based \n")
         st = self._index_checking(st)
@@ -172,6 +183,38 @@ class Fadict(dict):
     def getNames(self):
         return self.keys()
 
+class FadictIgnore(FadictBase, CaseInsensitiveDict):
+    """ Case unsensitive version of Fadict """
+    
+    def __new__(self,*args,ignorecase=False,**kwargs):
+        return CaseInsensitiveDict.__new__(self)
+
+    def __init__(self,infile,strict=False,ignorecase=False,**kwargs):
+        super(FadictIgnore,self).__init__()
+        self.init(infile,strict,ignorecase,**kwargs)
+
+class FadictCase(FadictBase, dict):
+    """ Case sensitive version of Fadict """
+    
+    def __new__(self,*args,ignorecase=False,**kwargs):
+        return dict.__new__(self)
+
+    def __init__(self,infile,strict=False,ignorecase=False,**kwargs):
+        self.init(infile,strict,ignorecase,**kwargs)
+
+def Fadict(*args,ignorecase=False,**kwargs):
+    """ A factory function to create Fadict objec, 
+    which provide a dictionary of sequences [seqname: seq].
+        :param: strict
+        :param: if_trim
+        :param: ignorecase
+    """
+
+    if ignorecase == True:
+        return FadictIgnore(*args,ignorecase,**kwargs)
+    else:
+        return FadictCase(*args,ignorecase,**kwargs)
+
 class Falist(list):
 
     def __new__(self,*args,**kwargs):
@@ -794,10 +837,16 @@ class GeneDict(SGENT):
         """
         return "\n".join(map(lambda x:x.as_gtf(),self.values()))
 
-    # Get representative transcript of one gene, usally the longest one.
-    # Useful when doing gene function annotation.
-    def get_present_tx(self):
-        pass
+    def get_represent_tx(self):
+        """ Get representative transcript of one gene, usally the longest one.
+            Useful when doing gene function annotation."""
+        max_len = 0 
+        for tx in self.values():
+            if len(tx) > max_len:
+                rep_tx = tx
+                max_len = len(tx)
+        return rep_tx
+
 
 class ChrDict(SGENT):
 
@@ -1418,6 +1467,22 @@ class GtfDict(GenomeDict):
     
     """ Read gtf file, convert to GenomeDict data.
         Only exon are used.
+
+        Example(Maybe correct):
+        genomedict = GtfDict(input_gtf_file)
+        # Get a geneDict object for gene named "EXMP"
+        dict_of_gene = genomedict.get_GeneDict()
+        gene = dict_of_gene.get("EXMP")
+        # Get information of the gene
+        print(gene.start)
+        print(gene.end)
+        print(gene.strand)
+        # Length of gene (sum of exon length), might be wrong, Fix in next version
+        print(len(gene))
+        # How many transcript in a gene
+        print(gene.count)
+
+
     """
     def __init__(self,infile,sep="\t",keeps=set(["exon"]),fm="normal"):
         """ fm[normal,gc,none]: format of gtf/gff file.
@@ -1442,6 +1507,9 @@ class GtfDict(GenomeDict):
             self.add_exon(EXON(rec))
 
     def get_GeneDict(self):
+        """ Get a dict of gene_id - geneDict pair.
+            {gene_id:GeneDict}
+        """
         data = Ordic()
         for mychr in self.values():
             for gene in mychr.values():
